#include "types.h"
#include "stdio.h"
#include "mc68302.h"
#include "intr.h"
#include "gpio.h"
#include "monitor_api.h"
#include "sdsl_asm.h"
#include "globals.h"

extern caddr_t m68000_vectors[];
extern volatile struct mc68302_regs mc68302_regs;
extern volatile long timebase;

extern char _Xvec_scc1;
extern char _Xvec_scc2;

extern struct sdcore_api *fetch_sdcore();
extern struct opcore_api *fetch_opcore();
extern struct l2conv_api *fetch_l2conv();
extern caddr_t malloc();

static const u_short appname[2] = {0x5C64, 0xC88B};

init()
{
	printf("\r\nATM/HDLC converting DSU (Flavor N1, SCC-based) code running\r\n");
	if (sdcore_init() < 0)
		return(-1);
	opcore_api = fetch_opcore();
	if (!opcore_api) {
		/* error message already printed */
		return(-1);
	}
	if (opcore_api->init(appname, sdcore_api, &sdcore_state) < 0)
		return(-1);
	l2conv_api = fetch_l2conv();
	if (!l2conv_api) {
		/* error message already printed */
		return(-1);
	}
	if (l2conv_api->init() < 0)
		return(-1);
	if (get_config() < 0)
		return(-1);
	l2conv_api->get_global_cfg(&l2conv_global);
	bzero(&atm_outgoing_queue, sizeof atm_outgoing_queue);
	nokia_init_idle_frame();
	p2f_init();
	if (opcore_api->setup_scc2() < 0)
		return(-1);
	m68000_vectors[INTVEC_SCC1] = &_Xvec_scc1;
	m68000_vectors[INTVEC_SCC2] = &_Xvec_scc2;
	bzero(&stats, sizeof stats);
	hdlc_init();
	eocdump_init();
	mc68302_regs.padat = (mc68302_regs.padat & ~PORTA_TXMUX_MASK) |
				PORTA_TXMUX_SCC1;
	register_custom_craft_commands();
	sdsl_asm_state = INACTIVE_STATE;
	timebase = 0;
	return(0);
}

sdcore_init()
{
	caddr_t privstate;

	sdcore_api = fetch_sdcore();
	if (!sdcore_api) {
		/* error message already printed */
		return(-1);
	}
	privstate = malloc(sdcore_api->sizeof_privstate);
	sdcore_api->swinit(&sdcore_state, privstate);
	return(0);
}
